@@ -30,7 +30,6 @@
     Symbol,
     SymbolFor,
     TypedArrayPrototypeSet,
-    SymbolToStringTag,
     TypeError,
     Uint8Array,
   } = window.__bootstrap.primordials;
@@ -359,10 +358,6 @@
       return bytes.buffer;
     }
 
-    get [SymbolToStringTag]() {
-      return "Blob";
-    }
-
     [SymbolFor("Deno.customInspect")](inspect) {
       return inspect(consoleInternal.createFilteredInspectProxy({
         object: this,
@@ -472,10 +467,6 @@
       webidl.assertBranded(this, File);
       return this[_LastModified];
     }
-
-    get [SymbolToStringTag]() {
-      return "File";
-    }
   }
 
   webidl.configurePrototype(File);
